#!/bin/sh
SERIAL_CONSOLE=ttyS0
SERIAL_DEVICE=/dev/${SERIAL_CONSOLE}
MY_TERMINAL=`tty`

function message
{
   echo $*
   if [ "$MY_TERMINAL" != "$SERIAL_DEVICE" ]; then
      echo $* > $SERIAL_DEVICE
   fi
}


BOOT_DEV=`mount | grep " on / type " | awk '{print $1}'`
if [ "$BOOT_DEV" != "/dev/mmcblk0p3" ]; then
   message "Update: No update - booted from $BOOT_DEV but expected /dev/mmcblk0p3"
   exit
fi

sleep 30

message "Update: Starting..."

rm -rf /run/media/mmcblk0p2/*
#cp -vpr /boot /ramdisk /run/media/mmcblk0p2
message "Update: Expanding rootfs"
tar -xf /deploy/arago-base-tisdk-image-ultrex.tar.xz -C /run/media/mmcblk0p2
cp -v /deploy/image-information.txt /run/media/mmcblk0p2/boot
rm /run/media/mmcblk0p2/usr/bin/mk-eMMC-boot.sh

./resize-partition.sh

mkdir -p /run/media/mmcblk0p4/app
ln -s /run/media/mmcblk0p4/app /run/media/mmcblk0p2/home/root/app
message "Update: Expanding application"
tar -xf /deploy/MegaLiveMotorControl.tar.bz2 -C /run/media/mmcblk0p4/app
chown -R root:root /run/media/mmcblk0p4/app

if [ -e /deploy/MLO.byteswap ]; then
   message "Update: Updating MLO"
   flash_erase /dev/mtd0 0 0
   flashcp /deploy/MLO.byteswap /dev/mtd0
fi

if [ -e /deploy/u-boot.img ]; then
   message "Update: Updating u-boot"
   flash_erase /dev/mtd1 0 0
   flashcp /deploy/u-boot.img /dev/mtd1
fi

echo 0 > /sys/block/mmcblk0boot1/force_ro
dd if=/dev/zero of=/dev/mmcblk0boot1 bs=512 count=8192

pushd /run/media/mmcblk0p4/app
if [ -e post_install.sh ]; then
   shutdown -r 4   # make sure the system reboots
   message "Update: Application"
   ./post_install.sh
fi
popd

message "Update: Complete"

sync

shutdown -r now

