#!/bin/sh

echo "`date`: $BASH_SOURCE" >> /var/log/application-start.log

if ! systemctl -q is-enabled watchdog; then
   echo "`date`: systemctl enable watchdog --now" >> /var/log/application-start.log
   systemctl enable watchdog --now &
elif ! systemctl -q status watchdog; then
   echo "`date`: systemctl start watchdog" >> /var/log/application-start.log
   systemctl start watchdog &
fi


# wait for app directory
for n in `seq 300`; do
   if [ -d /home/root/app ]; then
      break;
   fi
   echo "`date`: wait for app directory" >> /var/log/application-start.log
   sleep 1
done


# wait for slpd started
for n in `seq 120`; do
   echo "`date`: wait for slpd start" >> /var/log/application-start.log
   if [ -e /var/volatile/log/slpd.log ]; then
      SLPD_STARTED=1
      break;
   fi
   sleep 1
done


# wait for slpd ready
for n in `seq 120`; do
   echo "`date`: wait for slpd startup complete" >> /var/log/application-start.log
   grep -i "Startup complete" /var/volatile/log/slpd.log > /dev/null
   if [ "$?" == "0" ]; then
      SLPD_READY=1
      break
   fi
   sleep 1
done

if [ -z "$SLPD_STARTED" -o -z "$SLPD_READY" ]; then
   shutdown -r now
fi



function ApplicationWatchdog
# verify the application is running and restart if needed
{
   CURR_SECS=`date +%s`
   APPLICATION_COUNT=`/bin/pidof MegaLiveMotorControl | /usr/bin/wc -w`

   if [ ${APPLICATION_COUNT} -gt 0 ]; then
      APP_LAST_SEEN=${CURR_SECS}
   fi

   if [ ${APPLICATION_COUNT} -gt 1 ]; then
      MSG_DELTA_SECS=$((${CURR_SECS} - ${LAST_MESSAGE}))
      if [ ${MSG_DELTA_SECS} -gt 10 ]; then
         echo "Error: Applications running: ${APPLICATION_COUNT}" >> /dev/ttyS0
         LAST_MESSAGE=${CURR_SECS}
      fi
   fi

   APP_DELTA_SECS=$((${CURR_SECS} - ${APP_LAST_SEEN}))

   if [ ${APP_DELTA_SECS} -eq 5 ]; then
      echo "`date`: application not running" >> /var/log/application-start.log
   fi

   if [ ${APP_DELTA_SECS} -ge 60 ]; then
      echo "`date`: application starting" >> /var/log/application-start.log

      pushd /home/root/app
      ./MegaLiveMotorControl >> /var/log/MegaLiveMotorControl.log 2>&1 &
      popd

      APP_LAST_SEEN=${CURR_SECS}   # prevent restarting too often
   fi
}


function FailsafeWatchdog
# verify the failsafe is running and restart if needed
{
   CURR_SECS=`date +%s`
   FAILSAFE_COUNT=`/bin/pidof -x fail_safe.sh | /usr/bin/wc -w`

   if [ ${FAILSAFE_COUNT} -gt 0 ]; then
      FAILSAFE_LAST_SEEN=${CURR_SECS}
   fi

   if [ ${FAILSAFE_COUNT} -gt 1 ]; then
      MSG_DELTA_SECS=$((${CURR_SECS} - ${LAST_MESSAGE}))
      if [ ${MSG_DELTA_SECS} -gt 10 ]; then
         echo "Error: Watchdogs running: ${FAILSAFE_COUNT}" >> /dev/ttyS0
         LAST_MESSAGE=${CURR_SECS}
      fi
   fi

   FAILSAFE_DELTA_SECS=$((${CURR_SECS} - ${FAILSAFE_LAST_SEEN}))

   if [ ${FAILSAFE_DELTA_SECS} -eq 5 ]; then
      echo "`date`: failsafe not running" >> /var/log/application-start.log
   fi

   if [ ${FAILSAFE_DELTA_SECS} -ge 15 ]; then
      echo "`date`: failsafe starting" >> /var/log/application-start.log

      pushd /home/root/app
      ./fail_safe.sh &
      popd

      FAILSAFE_LAST_SEEN=${CURR_SECS}   # prevent restarting too often
   fi
}




APP_LAST_SEEN=0   #initialize to zero so first time will start the application
FAILSAFE_LAST_SEEN=0
LAST_MESSAGE=0


while true; do
   sleep 1
   ApplicationWatchdog
   FailsafeWatchdog
done
